<?php

/*
    daily_aux.inc - Part of the daily module for Drupal
    Copyright (c) 2004  Frodo Looijaard <drupal@frodo.looijaard.name>
                                                                                
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
                                                                                
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/** @file daily_aux.inc
 * Small, auxiliary functions called by other parts of the daily module
 */

/**************************
 * DATE RELATED FUNCTIONS *
 **************************/

/**
 * Pack a date object into a date string.
 *
 * A date object has members day, month and year.
 * A date string is of the form YYYYMMDD
 * @param $date The date object to convert.
 * @return The resulting date string
 */
function _daily_pack_date($date) {
  return sprintf("%04d%02d%02d", $date->year, $date->month, $date->day);
}

/**
 * Unpack a date string into a date object.
 *
 * A date object has members day, month and year. They are all proper numbers.
 * A date string is of the form YYYYMMDD
 * @param $date The date string to convert.
 * @return The resulting date object.
 */
function _daily_unpack_date($date) {
  $result->year = (int) substr($date, 0, 4);
  $result->month = (int) substr($date, 4, 2);
  $result->day = (int) substr($date, 6, 2);
  return $result;
}

/**
 * Get the current time in a date object.
 *
 * The date object is cached, so we only have to compute it once.
 * A date object has members year, month and day.
 * @return A date object with today's date.
 */
function _daily_today() {
  static $today;
  if (!$today) {
    $current_time = localtime(time(), True);
    $today->year = $current_time['tm_year'] + 1900;
    $today->month = $current_time['tm_mon'] + 1;
    $today->day = $current_time['tm_mday'];
  }
  return $today;
}


/**
 * Get the number of days of a certain month.
 *
 * @param $month The month (1 = Jan, 12 = Dec)
 * @param $year  The year  (for example, 2000)
 * @return The number of days is this particular month.
 * @remark Both $month and $year are needed to account for leap years.
 */
function _daily_days_per_month($month, $year) {
  switch ($month) {
    case 1: case 3: case 5: case 7: case 8: case 10: case 12:
      return 31;
    case 4: case 6: case 9: case 11: default:
      return 30;
    case 2:
      if ($year % 4 == 0) {
        return 29;
      }
      else {
        return 28;
      }
  }
}


/******************
 * ERROR MESSAGES *
 ******************/

/** 
 * Return an HTMLized error reflecting the taxonomy module is not enabled.
 *
 * @param $div: TRUE if a div should be generated, FALSE if not.
 * @return A string containing the HTML error.
 */
function _daily_form_error_no_taxonomy($div = FALSE) {
  return _daily_error_div(t("You do not seem to have the taxonomy module enabled. This is a requirement. You can enable it in %admin", array('%admin' => l(t("the modules administration page"), "admin/modules"))), $div);
}
                                                                                
/** 
 * Return an HTMLized error reflecting you have no daily taxonomy selected.
 *
 * @param $div: TRUE if a div should be generated, FALSE if not.
 * @return A string containing the HTML error.
 */
function _daily_form_error_no_vocabulary($div = FALSE) {
  return _daily_error_div(t("You do not seem to have a vocabulary assigned to daily items. This is a requirement. You can enable it in %admin", array('%admin' => l(t("the daily module administration page"), "admin/settings/daily"))), $div);
}
                                                                                
/** 
 * Return an HTMLized error reflecting you must select a daily taxonomy term.
 *
 * @param $div: TRUE if a div should be generated, FALSE if not.
 * @return A string containing the HTML error.
 */
function _daily_form_error_no_tid($div = FALSE) {
  return _daily_error_div(t("You have not selected a term in the taxonomy you assigned to daily items."), $div);
}

/** 
 * Return an HTMLized error reflecting you have to enter a proper number.
 *
 * @param $div: TRUE if a div should be generated, FALSE if not.
 * @return A string containing the HTML error.
 */
function _daily_form_error_correct_nr($div = FALSE) {
  return _daily_error_div(t("Please enter a whole number (positive or negative)."), $div);
}


/** 
 * Return an HTMLized error reflecting a daily item with this date already
 * exists.
 *
 * @param $div: TRUE if a div should be generated, FALSE if not.
 * @return A string containing the HTML error.
 */
function _daily_form_error_duplicate_date($div = FALSE) {
  return _daily_error_div(t("Duplicate date: you have to enter a unique date"), $div);
}


/**
 * Return an error string with optional HTML div around it
 *
 * @param $div: TRUE if a div should be generated, FALSE if not.
 * @return A string containing the HTML error.
 */ 
function _daily_error_div($output, $div = FALSE) {
  if ($div) {
    return "<div class=\"error\">". $output ."</div>";
  } 
  else {
    return $output;
  }
}

?>
