<?php

/*
    daily_container.inc - Part of the daily module for Drupal
    Copyright (c) 2004  Frodo Looijaard <drupal@frodo.looijaard.name>
                                                                                
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
                                                                                
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/** @file daily_container.inc
 * Functions specific for the node type daily-container
 *
 * All function names in this module are prepended with _daily_container
 */


/**
 * hook_access: Define access restrictions for node type daily-container
 *
 *   - view: Access is denied if this node is not yet published.
 *   - create: Access depends on the 'create daily items' permission
 *   - update: Access is allowed if you authored this node, and you have
 *             the 'edit own daily items' permission.
 *   - delete: Access is allowed if you authored this node, and you have
 *             the 'edit own daily items' permission.
 * @note See daily_perm() for the defined user permissions by this module.
 * @note See hook_access() for a description of parameters and return values.
 */
function _daily_container_access($op, $node) {
  global $user;
  if ($node->uid == 1) {
    return true;
  }
  switch ($op) {
    case 'create':
      return user_access('create daily items');
    case 'update':
      return user_access('edit own daily items') && ($user->uid == $node->uid);
    case 'delete':
      return user_access('edit own daily items') && ($user->uid == $node->uid);
  }
}

/**
 * hook_insert: Respond to node insertion for nodetype daily-container.
 *
 * Updates the database table {daily_container}.
 * @note See hook_insert() for a description of parameters and return values.
 */
function _daily_container_insert($node) {
  db_query("INSERT INTO {daily_container} (nid, tid, nr) VALUES (%d, %d, %d)", $node->nid, $node->tid, $node->nr);
}

/**
 * hook_update: Respond to node updating for nodetype daily-container
 *
 * Updates the database table {daily_container}
 * @note See hook_update() for a description of parameters and return values.
 */
function _daily_container_update($node) {
  db_query("UPDATE {daily_container} SET tid=%d, nr=%d WHERE nid=%d", $node->tid, $node->nr, $node->nid);
}


/**
 * hook_delete: Respond to node deletion for nodetype daily-container.
 *
 * Updates the database table {daily_container}
 * @note See hook_delete() for a description of parameters and return values.
 */
function _daily_container_delete(&$node) {
  db_query("DELETE FROM {daily_container} WHERE nid = %d", $node->nid);
}

/**
 * hook_load: Load node-type-specific information for nodetype daily-container.
 *
 * This is ugly. Load the contained daily node, unless we are editing (in which
 * case we load the real daily container node).
 *
 * If editing, the following field are added:
 *   - tid: The taxonomy term id of the daily collection for this container
 *   - nr: The offset within the collection (1 = first, -1 = last)
 *
 * In all other cases, the node object is modified to contain the daily node
 * to be displayed. The following fields are added:
 *   - container: The container node, which is the original node to which
 *                the same fields as above are added.
 *
 * @note See hook_load() for a description of parameters and return values.
 */
function _daily_container_load(&$node) {
  $data = db_fetch_object(db_query("SELECT tid, nr FROM {daily_container} WHERE nid = %d", $node->nid));

  // Ugly hack. If we are editing a node, we need the container itself.
  if (arg(2) == "edit") {
   return $data;
  }
  
  $oldnode = $node;
  $oldnote->tid = $data->tid;
  $oldnote->nr = $data->nr;
  $node = _daily_node_by_tid($data->tid, $data->nr);
  $node->container = $oldnode;
  $node->title = $oldnode->title;
}


/**
 * hook_validate: Verify a node editing form for nodetype daily-container.
 *
 * Note that some things can only be verified after a POST (preview/submit).
 * All input is validated strictly.
 * @note See hook_validate() for a description of parameters and return values.
 */
function _daily_container_validate(&$node, $vid) {
  if (array_key_exists('edit', $_POST)) {
    $term = taxonomy_get_term($node->tid);
    if (!$node->tid or !$term or ($term->vid != $vid)) {
      form_set_error('tid', _daily_form_error_no_tid());
    }
    if (!$node->nr or $node->nr !== (string) (int) $node->nr or $node->nr != floor($node->nr)) {
      form_set_error('nr', _daily_form_error_correct_nr());
    }
  }
}

/**
 * hook_form: Display a node editing form for nodetype daily-container.
 *
 * @note See hook_form() for a description of parameters and return values.
 */
function _daily_container_form(&$node, &$param) {

  $vid = variable_get("daily_vocabulary", 0);
  $query = db_query("SELECT term_data.tid,term_data.name FROM {term_data} WHERE term_data.vid = %d ORDER BY term_data.name", $vid);
                                                                                
  while ($db_record = db_fetch_object($query)) {
    $options[$db_record->tid] = $db_record->name;
  }
                                                                                
  $output .= form_select(t("Vocabulary term"), 'tid', $node->tid, $options, t("The daily item class"), 0, FALSE, TRUE);
                                                                                
  $output .= form_textfield(t("Position"), "nr", $node->nr, 3, 5, t("The position within the class. Use -1 for the last item, -2 for the last-but-one item, etc; use 1 for the first item, 2 for the second item, etc."), NULL, TRUE);

 $output .= form_textarea(t('Log message'), 'log', $node->log, 60, 5, t('An explanation of the additions or updates being made to help other authors understand your motivations.'));
                                                                                
  return $output;
}

/**
 * hook_view: View a node for nodetype daily-container.
 *
 * Note that this will only be called when we are previewing!
 *
 * @note See hook_view() for a description of parameters and return values.
 */
function _daily_container_view(&$node, $teaser = FALSE, $page = FALSE) {
  $node->body = "<i>A daily node will be displayed here</i>";
}

?>
