<?php

/*
    daily_daily.inc - Part of the daily module for Drupal
    Copyright (c) 2004  Frodo Looijaard <drupal@frodo.looijaard.name>
                                                                                
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
                                                                                
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/** @file daily_daily.inc
 * Functions specific for the node type daily-daily
 *
 * All function names in this module are prepended with _daily_daily
 */


/**
 * hook_access: Define access restrictions for node type daily-daily
 *
 *   - view: Access is denied if this node is not yet published.
 *           Access is denied if this node's date is in the future and
 *           you do not have the 'view future daily items' permission.
 *   - create: Access depends on the 'create daily items' permission
 *   - update: Access is allowed if you authored this node, and you have
 *             the 'edit own daily items' permission.
 *   - delete: Access is allowed if you authored this node, and you have
 *             the 'edit own daily items' permission.
 * @note See daily_perm() for the defined user permissions by this module.
 * @note See hook_access() for a description of parameters and return values.
 */
function _daily_daily_access($op, $node) {
  global $user;
  switch ($op) {
    case 'create':
      return user_access('create daily items');
    case 'update':
      return user_access('edit own daily items') && ($user->uid == $node->uid);
    case 'delete':
      return user_access('edit own daily items') && ($user->uid == $node->uid);
  }
}

/**
 * hook_insert: Respond to node insertion for nodetype daily-daily.
 *
 * Updates the database table {daily}.
 * @note See hook_insert() for a description of parameters and return values.
 */
function _daily_daily_insert($node) {
 db_query("INSERT INTO {daily} (nid, date, autopublish) VALUES (%d, '%s', %d)", $node->nid, _daily_pack_date($node), $node->autopublish);
}  

/**
 * hook_update: Respond to node updating for nodetype daily-daily.
 *
 * Updates the database table {daily}
 * @note See hook_update() for a description of parameters and return values.
 */
function _daily_daily_update($node) {
  db_query("UPDATE {daily} SET date='%s', autopublish=%d WHERE nid=%d", _daily_pack_date($node), $node->autopublish, $node->nid);
}

/**
 * hook_delete: Respond to node deletion for nodetype daily-daily.
 *
 * Updates the database table {daily}
 * @note See hook_delete() for a description of parameters and return values.
 */
function _daily_daily_delete(&$node) {
  db_query("DELETE FROM {daily} WHERE nid = %d", $node->nid);
}


/**
 * hook_load: Load node-type-specific information for nodetype daily-daily.
 *
 * Adds the following fields to the node object:
 *   - year: The year of the node date (int)
 *   - month: The month of the node date (int, 1..12)
 *   - day: The day of the node date (int, 1..31)
 *   - tid: The taxonomy term id within the vocabulary associated with daily.
 *   - term_name: The taxonomy term name of tid.
 * @note See hook_load() for a description of parameters and return values.
 */
function _daily_daily_load($node) {
  $db_record = db_fetch_object(db_query("SELECT d.date, d.autopublish, t.tid, td.name FROM {daily} d,{term_node} t,{term_data} td WHERE d.nid = %d AND t.nid = d.nid AND td.tid = t.tid", $node->nid));
  $data = _daily_unpack_date($db_record->date);
  $data->tid = $db_record->tid;
  $data->term_name = $db_record->name;
  $data->autopublish = $db_record->autopublish;
  return $data;
}

/**
 * hook_validate: Verify a node editing form for nodetype daily-daily.
 *
 * Note that some things can only be verified after a POST (preview/submit).
 * All input is validated strictly.
 * @note See hook_validate() for a description of parameters and return values.
 */
function _daily_daily_validate(&$node, $vid) {
  _daily_sanitize_date($node);
                                                                                
  /* Not so nice. We have to find our tid. But all we have is the taxonomy
   * field in our node. It is an array of tids. Presumably, one belongs to
   * us. But which one?
   * To make things worse, if the daily taxonomy has not been defined
   * properly, we may not have gotten a tid at all. But we have to
   * differentiate that from the case where there has been no POST yet.
   * So we use a heuristic here. */
  if (array_key_exists('edit', $_POST)) {
    $tid = _daily_find_our_tid($node->taxonomy);
    if ($tid) {
    /* Check for a duplicate. It is a duplicate if it has the same date and
     * a different node id, provided it has the same taxonomy term id */
      if (db_result(db_query("SELECT * from {daily} d,{term_node} t WHERE d.nid = t.nid AND d.date = '%s' AND t.tid = %d AND d.nid <> %d", _daily_pack_date($node), $tid, $node->nid))) {
        form_set_error('pubdate', _daily_form_error_duplicate_date());
      }
      $node->tid = $tid;
      $data = db_fetch_object(db_query("SELECT name FROM {term_data} WHERE tid = %d", $tid));
      $node->term_name = $data->name;
    }
    else {
      form_set_error('warning', _daily_form_error_no_tid());
      $node->tid = 0;
      $node->term_name = t("UNASSIGNED");
    }
    if ($node->autopublish) {
      $node->status = _daily_pack_date($node) <= _daily_pack_date(_daily_today())?1:0;
     
    }
  }
}

                                                                             
/**
 * hook_form: Display a node editing form for nodetype daily-daily.
 *
 * @note See hook_form() for a description of parameters and return values.
 */
function _daily_daily_form(&$node, &$param) {
  _daily_sanitize_date($node);
                                                                                
  $output = "";
  $output .= implode("", taxonomy_node_form("daily-daily", $node));
                                                                                
  $group = "";
  $group .= form_select(t("Day"), 'day', $node->day, drupal_map_assoc(range(1, 31)), t("The publication day"), 0, FALSE, TRUE);
  $group .= form_select(t("Month"), 'month', $node->month, array(1 => t("January"), t("February"), t("March"), t("April"), t("May"), t("June"), t("July"), t("August"), t("September"), t("October"), t("November"), t("December")), t("The publication month"), 0, FALSE, TRUE);
  $group .= form_select(t("Year"), 'year', $node->year, drupal_map_assoc(range(DAILY_MIN_YEAR, DAILY_MAX_YEAR)), t("The publication year"), 0, FALSE, TRUE);
  $output .= form_group(t("Date"), $group, t("The publication date"));
  $output .= form_checkbox(t("Publish automatically"), 'autopublish', 1, (isset($node->autopublish)?$node->autopublish:variable_get("daily_default_autopublish", 1)), t("Publish this note automatically on the selected date"));

  $output .= form_textarea(t("Body"), 'body', $node->body, 60, 20, '', NULL, TRUE);

  $output .= filter_form('format', $node->format);
  $output .= form_textarea(t('Log message'), 'log', $node->log, 60, 5, t('An explanation of the additions or updates being made to help other authors understand your motivations.'));

  return $output;
}

/**
 * hook_view: View a node for nodetype daily-daily.
 *
 * @note See hook_view() for a description of parameters and return values.
 */
function _daily_daily_view(&$node, $teaser = FALSE, $page = FALSE) {
  _daily_content($node, $teaser, $page);
}

?>
