<?php
                                                                                
/*
    daily_data.inc - Part of the daily module for Drupal
    Copyright (c) 2004  Frodo Looijaard <drupal@frodo.looijaard.name>
                                                                                
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
                                                                                
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/** @file daily_data.inc
 * Functions specific for the generation and display of data
 */

/*******************************************
 * DAILY-DAILY AND DAILY-CONTAINER DISPLAY *
 *******************************************/

/**
 * Add a link to the daily CSS sheet
 *
 * This function adds a link in the output HTML to the CSS sheet belonging to
 * the daily module. It makes sure the link is only added once.
 * @note Note that this is mostly a hack; we may have to think of some better
 * way to solve this!
 */
function _daily_css() {
  static $daily_css_added;
  if (!$daily_css_added) {
    drupal_set_html_head("<link type=\"text/css\" rel=\"stylesheet\" href=\"modules/daily/daily.css\" />");
    $daily_css_added = 1;
  }
}


/**
 * Find the taxonomy term id that belongs to the daily vocabulary
 *
 * @param $tids An array of tids
 * @result The found tid, or undef if not found.
 */
function _daily_find_our_tid($tids) {
  $vid = variable_get("daily_vocabulary", "");
  foreach ($tids as $tid) {
    $term = taxonomy_get_term($tid);
    if ($term->vid == $vid) {
      return $tid;
    }
  }
}

/**
 * Retrieve the node of a daily item associated with a tid on a certain offset
 *
 * @param $tid Taxonomy term identifier
 * @param $number Number of the page to display. $number is negative to count
 * from the start (0=-1=last), and positive to count from the start (0=first).
 * @param $container_nid The nid of the container that contains this page;
 *                       0 means no container.
 * @return The fully filled node object found (or nothing if not found).
 */
function _daily_node_by_tid($tid, $number) {
  $sql_extra = user_access('administer nodes')?"":" AND n.status = 1";
  if ($tid != (string) (int) $tid or $tid != floor($tid)) {
    return;
  }
  if (!$number) {
    $number = 1;
  }
  if ($number != (string) (int) $number or $number != floor($number)) {
    return;
  }
  $query = db_query("SELECT t.nid FROM {term_node} t,{daily} d, {node} n WHERE t.tid = %d AND t.nid = d.nid AND n.nid = d.nid". $sql_extra ." ORDER BY d.date %s LIMIT %d,1", $tid, ($number < 0?"DESC":""), abs($number) - 1);
  if (!($db_record = db_fetch_array($query))) {
    // If we are out of bound, return the first or last record.
    $query = db_query("SELECT t.nid FROM {term_node} t,{daily} d, {node} n WHERE t.tid = %d AND t.nid = d.nid AND n.nid = d.nid". $sql_extra ." ORDER BY d.date %s LIMIT 0,1", $tid, ($number > 0?"DESC":""));
    if (!($db_record = db_fetch_array($query))) {
      return;
    }
  }
  $node = node_load($db_record);
  return $node;
}

/**
 * View a node of type daily-daily
 *
 * @param $node The completely loaded node
 * @param $teaser 1 if only a teaser should be displayed, 0 for the full page
 * @param $page 0 if the title should be displayed, 1 if not.
 */
function _daily_content(&$node, $teaser, $page) {
  $node = node_prepare($node, $teaser);
  _daily_css();
  $output = theme("daily_header", $node);
  $node->body = $output . $node->body;
  $node->teaser = $output . $node->teaser;
  $node->body .= theme("daily_calendar_month", _daily_calendar_month_data($node->tid, $node));
}

/**
 * Compute the data needed to display a month calendar with links
 *
 * @param $tid Taxonomy term ID for this daily item collection
 * @param $date The current date in date object format.
 * @return An array with the following elements:
 *         For each day of this month, an element with the day number as key.
 *         Each array element is again an array, with a key 'weekday'
 *         containing the day of the week (sunday=0, saterday=6); with
 *         optionally a key 'link' containing the URL to link to (only if
 *         there is a daily item); and with optionally a key 'today' containing
 *         True if this is today; and a key 'future' which is True if the
 *         date lies in the future, and False if not.
 *         The return array also has elements with keys 'next' and 'prev'
 *         containing the URLs for the next and previous month; if there
 *         is no next or previous month, the corresponding element is not
 *         included.
 *         The return array also has an element 'date' containing the
 *         current date in system notation (seconds after Jan 1, 1980)
 *         So we have something like:
 *           $result[1]['weekday']  = 5
 *           $result[1]['link']     = 'daily/tid/4/20030101'
 *           $result[1]['today']    = True
 *           $result[1]['future']   = False
 *           .....
 *           $result['prev']        = 'daily/tid/4/20021231'
 *           $result['next']        = 'daily/tid/4/20030203'
 *           $result['date']        = 94758292343
 */
function _daily_calendar_month_data($tid, $date) {
  $result = array();
  $sql_extra = user_access('administer nodes')?"":" AND n.status = 1";

  // Calculate weekdays and whether dates lie in the future.
  $start_of_month = mktime(0, 0, 0, $date->month, 1, $date->year);
  $first_day = date("w", $start_of_month);
  $today_packed = _daily_pack_date(_daily_today());
  for ($day = 1; $day <= _daily_days_per_month($date->month, $date->year); $day++) {
    $result[$day]["weekday"] = ($first_day + $day - 1) % 7;
    $this_day = $date;
    $this_day->day = $day;
    $result[$day]["future"] = _daily_pack_date($this_day) > $today_packed;
  }

  // Enter all dates in the current month which have items associated with them
  $som = $date;
  $som->day = 0;
  $eom = $date;
  $eom->day = 99;
  $query = db_query("SELECT d.date,d.nid FROM {term_node} t,{daily} d, {node} n WHERE t.tid = %d AND t.nid = d.nid AND n.nid = d.nid AND d.date >= '%s' AND d.date < '%s' ". $sql_extra, $tid, _daily_pack_date($som), _daily_pack_date($eom));
  while ($db_record = db_fetch_object($query)) {
    $found_date = _daily_unpack_date($db_record->date);
    $result[$found_date->day]["link"] = "node/". $db_record->nid;
  }
                                                                                
  // Mark the selected date, and enter the selected month.
  $result[$date->day]["today"] = True;
  $result["date"] = mktime(0, 0, 0, $date->month, 15, $date->year);
                                                                                
  // Calculate the link to the previous month
  $query = db_query("SELECT d.nid FROM {term_node} t,{daily} d,{node} n WHERE t.tid = %d AND t.nid = d.nid AND n.nid = d.nid AND d.date < '%s' ". $sql_extra ." ORDER BY d.date DESC", $tid, _daily_pack_date($som));
  if ($db_record = db_fetch_object($query)) {
    $result['prev'] = "node/". $db_record->nid;
  }

  // Calculate the link to the next month
  $query = db_query("SELECT d.nid FROM {term_node} t,{daily} d,{node} n WHERE t.tid = %d AND t.nid = d.nid AND n.nid = d.nid AND d.date > '%s' ". $sql_extra ." ORDER BY d.date", $tid, _daily_pack_date($eom));
  if ($db_record = db_fetch_object($query)) {
    $result['next'] = "node/". $db_record->nid;
  }
                                                                                
  return $result;
}


/*******************
 * THEME FUNCTIONS *
 *******************/
                                                                                
/**
 * Theme the extra header of a daily-daily node
 *
 * @param $node The node data which is used for the theming
 * @return A string which can be printed.
 */
function theme_daily_header($node) {
  return  "<h2 class=\"daily-header\">". $node->term_name .", ". format_date(mktime(12, 0, 0, $node->month, $node->day, $node->year), 'custom', "j F Y") ."</h2>";
}
                                                                                
                                                                                
/**
 * Theme a month calendar, using data as returned by _daily_calendar_month_data
 *
 * @param $data The data returned by _daily_calendar_month_data
 * @return A string which can be printed.
 */
function theme_daily_calendar_month($data) {
  $var_first_day = variable_get("default_firstday", 0);
  $output = "<div class=\"daily-calendar\">\n";
  $output .= "<table summary=\"". t('A calendar to browse through this daily item collection') .".\">\n";
  $output .= "<tr><td colspan=\"7\" class=\"header-month\">". (array_key_exists('prev', $data)?l("&laquo;", $data["prev"]):"&nbsp;") ." ". format_date($data["date"], 'custom', "F Y") ." ".  (array_key_exists('next', $data)?l("&raquo;", $data["next"]):"&nbsp;") ."</td></tr>\n";
                                                                                
  $output .= "<tr class=\"header-week\">";
  $days = array(t('Su'), t('Mo'), t('Tu'), t('We'), t('Th'), t('Fr'), t('Sa'));
  for ($i = $var_first_day; $i < $var_first_day + 7; $i++)
    $output .= "<td>". $days[$i % 7] ."</td>";
  $output .= "</tr>\n";
                                                                                
  $first_day = ((1 - $data[1]["weekday"] + $var_first_day));
  while ($first_day > 1)
    $first_day -= 7;
  while ($first_day < -5)
    $first_day += 7;
  $dow = 0;
  for ($day = $first_day, $dow = 0; $day < 99; $day++, $dow = ($dow + 1) % 7) {
    if ($dow == 0 and $day >= 1 and !array_key_exists($day, $data)) {
      break;
    }
    if ($dow == 0) {
      $output .= "<tr class=\"row-week\">";
    }
    if (!array_key_exists($day, $data)) {
      $output .= "<td class=\"day-blank\"></td>";
    }
    else if (array_key_exists("today", $data[$day])) {
      $output .= "<td class=\"day-today\"><div>". $day ."</div></td>";
    }
    elseif (array_key_exists("link", $data[$day])) {
      $output .= "<td class=\"day-link\">". l($day, $data[$day]["link"]) ."</td>";
    }
    elseif ($data[$day]["future"]) {
      $output .= "<td class=\"day-future\"><div>". $day ."</div></td>";
    }
    else {
      $output .= "<td class=\"day-normal\"><div>". $day ."</div></td>";
    }
    if ($dow == 6) {
      $output .= "</tr>\n";
    }
  }
  $output .= "</table></div>\n";
  return $output;
}
